/********************************************************************
 FileName:	   main.c
 Dependencies: See INCLUDES	section
 Processor:		PIC18 or PIC24 USB Microcontrollers
 Hardware:		The	code is	natively intended to be	used on	the	following
				hardware platforms:	PICDEM FS USB Demo Board, 
				PIC18F87J50	FS USB Plug-In Module, or
				Explorer 16	+ PIC24	USB	PIM.  The firmware may be
				modified for use on	other USB platforms	by editing the
				HardwareProfile.h file.
 Complier:		Microchip C18 (for PIC18) or C30 (for PIC24)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith	by Microchip Technology	Incorporated
 (the Company for its PIC(TM) Microcontroller is intended and
 supplied to you, the Company's	customer, for use solely and
 exclusively on	Microchip PIC Microcontroller products.	The
 software is owned by the Company and/or its supplier, and is
 protected under applicable	copyright laws.	All	rights are reserved.
 Any use in	violation of the foregoing restrictions	may	subject	the
 user to criminal sanctions	under applicable laws, as well as to
 civil liability for the breach	of the terms and conditions	of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS	IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY,	INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES	OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR	PURPOSE	APPLY TO THIS SOFTWARE.	THE	COMPANY	SHALL NOT,
 IN	ANY	CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES,	FOR	ANY	REASON WHATSOEVER.

********************************************************************
 File Description:

 Change	History:
  Rev	Date		 Description
  1.0	11/19/2004	 Initial release
  2.1	02/26/2007	 Updated for simplicity	and	to use common
					 coding	style
********************************************************************/

/**	INCLUDES *******************************************************/
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "usb_config.h"
#include "./USB/usb_device.h"
#include "./USB/usb.h"
#include "./USB/usb_function_cdc.h"
#include "HardwareProfile.h"
#include "eusart.h"


#include "fuse-config.h"

//ő{[[gw.
//#define		MAX_BAUDRATE	115200

//1Ȃ, USARTgpȂ echoback ݂̂s.
#define	ECHOBACK_TEST	(0)

/**	V A	R I	A B	L E	S ********************************************************/

#pragma udata access accessram

uchar 	near dummyWork;
uchar	near LastRS232Out;  // Number of	characters in the buffer
uchar	near RS232cp;	   // current position within the buffer


#pragma	udata
char 	USB_Out_Buffer[CDC_DATA_OUT_EP_SIZE];
char 	RS232_Out_Data[CDC_DATA_IN_EP_SIZE];

uchar	NextUSBOut;
//uchar	RS232_Out_Data_Rdy = 0;
USB_HANDLE		lastTransmission;

/**	P R	I V	A T	E  P R O T O T Y P E S ***************************************/
#if 0
static void	InitializeSystem(void);
#endif

void ProcessIO(void);
void USBDeviceTasks(void);
void BlinkUSBStatus(void);
void UserInit(void);


/**	DECLARATIONS ***************************************************/
#pragma	code


#if	USART_USE_INTERRUPT			// 荞݂gp.
	char    mDataRdyUSART(void);
	#define	mTxRdyUSART()	TXSTAbits.TRMT
#else
	#define	mDataRdyUSART()	PIR1bits.RCIF
	#define	mTxRdyUSART()	TXSTAbits.TRMT
#endif



void InitializeUSART(void)
{
	uchar i;
//	Initialize	the	arrays
	for	(i=0; i<sizeof(USB_Out_Buffer);	i++){
		USB_Out_Buffer[i] =	0;
	}

	NextUSBOut = 0;
	LastRS232Out = 0;
	lastTransmission = 0;

	initUSART();
}
/******************************************************************************
 * Function:		void mySetLineCodingHandler(void)
 *
 * PreCondition:	USB_CDC_SET_LINE_CODING_HANDLER	is defined
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function gets called when a SetLineCoding command
 *					is sent	on the bus.	 This function will	evaluate the request
 *					and	determine if the application should	update the baudrate
 *					or not.
 *
 * Note:			
 *
 *****************************************************************************/
#if	defined(USB_CDC_SET_LINE_CODING_HANDLER)
void mySetLineCodingHandler(void)
{
#if		MAX_BAUDRATE
	//If the request is	not	in a valid range
	if(cdc_notice.GetLineCoding.dwDTERate.Val >	MAX_BAUDRATE)
	{
		//NOTE:	There are two ways that	an unsupported baud	rate could be
		//handled.	The	first is just to ignore	the	request	and	don't change
		//the values.  That	is what	is currently implemented in	this function.
		//The second possible method is	to stall the STATUS	stage of the request.
		//STALLing the STATUS stage	will cause an exception	to be thrown in	the	
		//requesting application.  Some	programs, like HyperTerminal, handle the
		//exception	properly and give a	pop-up box indicating that the request
		//settings are not valid.  Any application that	does not handle	the
		//exception	correctly will likely crash	when this requiest fails.  For
		//the sake of example the code required	to STALL the status	stage of the
		//request is provided below.  It has been left out so that this	demo
		//does not cause applications without the required exception handling
		//to crash.
		//---------------------------------------
		//USBStallEndpoint(0,1);
	}else
#endif
	{
		DWORD_VAL dwBaud;

		//Update the baudrate info in the CDC driver
		CDCSetBaudRate(cdc_notice.GetLineCoding.dwDTERate.Val);
		
		//Update the baudrate of the UART
		#if	defined(__18CXX)
			dwBaud.Val = (GetSystemClock()/4)/line_coding.dwDTERate.Val-1;
			SPBRG  = dwBaud.v[0];
			SPBRGH = dwBaud.v[1];
		#endif
	}
}

#if	ECHOBACK_TEST
/********************************************************************
 * Function:		void ProcessIO(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function is a place holder	for	other user
 *					routines. It is	a mixture of both USB and
 *					non-USB	tasks.
 *
 * Note:			None
 *******************************************************************/

//	----------------------------------------
// 			GR[obNeXg
//	----------------------------------------

void ProcessIO(void)
{	
	//Blink	the	LEDs according to the USB device status
	BlinkUSBStatus();
	// User	Application	USB	tasks
	if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))	return;

	// only check	for	new	USB	buffer if the old RS232	buffer is
	// empty.	 This will cause additional	USB	packets	to be NAK'd

	// RS232_Mobt@ HostPC ͂f[^Fill.
	if (LastRS232Out == 0) {						  
		LastRS232Out = getsUSBUSART(RS232_Out_Data,64);	//until	the	buffer is free.
		if(LastRS232Out	> 0) {
//			RS232_Out_Data_Rdy = 1;	 //	signal buffer full
			RS232cp	= 0;  // Reset the current position
		}
	}

	if(LastRS232Out && (NextUSBOut==0)) {
		//GR[obNp:
		memcpy(USB_Out_Buffer,RS232_Out_Data,LastRS232Out);
		NextUSBOut = LastRS232Out;
		LastRS232Out = 0;
	}

	//\ȂARS232_Mobt@̓eHostPCɑ.
	if((USBUSARTIsTxTrfReady())	&& (NextUSBOut > 0)){
		putUSBUSART(&USB_Out_Buffer[0],	NextUSBOut);
		NextUSBOut = 0;
	}

	CDCTxService();
}

//	GR[obN͂܂.

#else


#if	USART_USE_INTERRUPT		// 荞݂gp.

//	----------------------------------------
// 			荞݂gp.
//	----------------------------------------

void ProcessIO(void)
{	
	uchar i;
	BlinkUSBStatus();		//Blink	the	LEDs according to the USB device status
	// User	Application	USB	tasks
	if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))	return;

	// only check	for	new	USB	buffer if the old RS232	buffer is
	// empty.	 This will cause additional	USB	packets	to be NAK'd

	// RS232_Mobt@ HostPC ͂f[^Fill.
	if( mDataRdyUSART() < 32 ) 

	if (LastRS232Out == 0) {						  
		LastRS232Out = getsUSBUSART(RS232_Out_Data,64);	//until	the	buffer is free.
		if(LastRS232Out	> 0) {
//			RS232_Out_Data_Rdy = 1;	 //	signal buffer full
			RS232cp	= 0;  // Reset the current position
		}
	}

#if	USART_USE_TX_INTERRUPT		// M荞݂gp.
	//USARTMł RS232_Mobt@̓e USART P.
	if(LastRS232Out && mTxRdyUSART())	{
//		PIR1bits.TXIF = 0;			// M荞݃tO.
		PIE1bits.TXIE = 1;			// M荞݋.
	}
#else
	//USARTMł RS232_Mobt@̓e USART P.
	if(LastRS232Out && mTxRdyUSART())	{
		putcUSART(RS232_Out_Data[RS232cp]);	//P.
		++RS232cp;
		if (RS232cp	== LastRS232Out) {
//			RS232_Out_Data_Rdy = 0;
			LastRS232Out=0;
			RS232cp=0;
		}
	}
#endif

	//\ȂARS232_Mobt@̓eHostPCɑ.
	if((USBUSARTIsTxTrfReady())	&& (mDataRdyUSART())){
		i=0;
		while( mDataRdyUSART() ) {
			if(i>=CDC_DATA_OUT_EP_SIZE) break;
			USB_Out_Buffer[i++]=getcUSART();
		}
		putUSBUSART(&USB_Out_Buffer[0],	i);
		NextUSBOut = 0;
	}

	CDCTxService();
}

#else

//	----------------------------------------
// 			荞݂gpȂ.
//	----------------------------------------

void ProcessIO(void)
{	
	uchar cnt;
	//Blink	the	LEDs according to the USB device status
	BlinkUSBStatus();
	// User	Application	USB	tasks
	if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))	return;

	// only check	for	new	USB	buffer if the old RS232	buffer is
	// empty.	 This will cause additional	USB	packets	to be NAK'd

	// RS232_Mobt@ HostPC ͂f[^Fill.
	if (LastRS232Out == 0) {						  
		LastRS232Out = getsUSBUSART(RS232_Out_Data,64);	//until	the	buffer is free.
		if(LastRS232Out	> 0) {
			RS232cp	= 0;  // Reset the current position
		}
	}

	for(cnt=0;cnt<32;cnt++) {

		//USARTMł RS232_Mobt@̓e USART P.
		if(LastRS232Out && mTxRdyUSART())	{
			putcUSART(RS232_Out_Data[RS232cp]);	//P.
			++RS232cp;
			if (RS232cp	== LastRS232Out) {
				LastRS232Out=0;
				RS232cp=0;
			}
		}

		//USARTf[^Mς݂ł RS232_Mobt@ɗ߂.
		if(mDataRdyUSART()) {
			USB_Out_Buffer[NextUSBOut] = getcUSART();
			if(	NextUSBOut< (CDC_DATA_OUT_EP_SIZE-1) ) {	//2011-7-10ǉ.
				NextUSBOut++;
			}												//2011-7-10ǉ.
			//USB_Out_Buffer[NextUSBOut] = 0;
		}

	}

	//\ȂARS232_Mobt@̓eHostPCɑ.
	if((USBUSARTIsTxTrfReady())	&& (NextUSBOut > 0)){
		putUSBUSART(&USB_Out_Buffer[0],	NextUSBOut);
		NextUSBOut = 0;
	}

	CDCTxService();
}

#endif
#endif

/********************************************************************
 * Function:		void BlinkUSBStatus(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		BlinkUSBStatus turns on	and	off	LEDs 
 *					corresponding to the USB device	state.
 *
 * Note:			mLED macros	can	be found in	HardwareProfile.h
 *					USBDeviceState is declared and updated in
 *					usb_device.c.
 *******************************************************************/
void BlinkUSBStatus(void)
{
	static WORD	led_count=0;
	
	if(led_count ==	0)led_count	= 10000U;
	led_count--;

	#define	mLED_Both_Off()			{mLED_1_Off();mLED_2_Off();}
	#define	mLED_Both_On()			{mLED_1_On();mLED_2_On();}
	#define	mLED_Only_1_On()		{mLED_1_On();mLED_2_Off();}
	#define	mLED_Only_2_On()		{mLED_1_Off();mLED_2_On();}

	if(USBSuspendControl ==	1)
	{
		if(led_count==0)
		{
			mLED_1_Toggle();
			if(mGetLED_1())
			{
				mLED_2_On();
			}
			else
			{
				mLED_2_Off();
			}
		}//end if
	}
	else
	{
		if(USBDeviceState == DETACHED_STATE)
		{
			mLED_Both_Off();
		}
		else if(USBDeviceState == ATTACHED_STATE)
		{
			mLED_Both_On();
		}
		else if(USBDeviceState == POWERED_STATE)
		{
			mLED_Only_1_On();
		}
		else if(USBDeviceState == DEFAULT_STATE)
		{
			mLED_Only_2_On();
		}
		else if(USBDeviceState == ADDRESS_STATE)
		{
			if(led_count ==	0)
			{
				mLED_1_Toggle();
				mLED_2_Off();
			}//end if
		}
		else if(USBDeviceState == CONFIGURED_STATE)
		{
			if(led_count==0)
			{
				mLED_1_Toggle();
				if(mGetLED_1())
				{
					mLED_2_Off();
				}
				else
				{
					mLED_2_On();
				}
			}//end if
		}//end if(...)
	}//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus


// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware	stack will call	the	callback functions USBCBxxx() in response to certain USB related
// events.	For	example, if	the	host PC	is powering	down, it will stop sending out Start of	Frame (SOF)
// packets to your device.	In response	to this, all USB devices are supposed to decrease their	power
// consumption from	the	USB	Vbus to	<2.5mA each.  The USB module detects this condition	(which according
// to the USB specifications is	3+ms of	no bus activity/SOF	packets) and then calls	the	USBCBSuspend()
// function.  You should modify	these callback functions to	take appropriate actions for each of these
// conditions.	For	example, in	the	USBCBSuspend(),	you	may	wish to	add	code that will decrease	power
// consumption from	Vbus to	<2.5mA (such as	by clock switching,	turning	off	LEDs, putting the
// microcontroller to sleep, etc.).	 Then, in the USBCBWakeFromSuspend() function, you may then	wish to
// add code	that undoes	the	power saving things	done in	the	USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack	will not automatically call	this
// function.  This function	is meant to	be called from the application firmware	instead.  See the
// additional comments near	the	function.

/******************************************************************************
 * Function:		void USBCBSuspend(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		Call back that is invoked when a USB suspend is	detected
 *
 * Note:			None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power	saving code.  Insert appropriate code here for the desired
	//application behavior.	 If	the	microcontroller	will be	put	to sleep, a
	//process similar to that shown	below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable	at least USBActivityIF as a	wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this	should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this	should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT	NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.	 This bit is 
	//cleared inside the usb_device.c file.	 Clearing USBActivityIF	here will cause	
	//things to	not	work as	intended.	
	

	#if	defined(__C30__)
	#if	0
		U1EIR =	0xFFFF;
		U1IR = 0xFFFF;
		U1OTGIR	= 0xFFFF;
		IFS5bits.USB1IF	= 0;
		IEC5bits.USB1IE	= 1;
		U1OTGIEbits.ACTVIE = 1;
		U1OTGIRbits.ACTVIF = 1;
		Sleep();
	#endif
	#endif
}


/******************************************************************************
 * Function:		void _USB1Interrupt(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function is called	when the USB interrupt bit is set
 *					In this	example	the	interrupt is only used when	the	device
 *					goes to	sleep when it receives a USB suspend command
 *
 * Note:			None
 *****************************************************************************/
#if	0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
	#if	!defined(self_powered)
		if(U1OTGIRbits.ACTVIF)
		{
			IEC5bits.USB1IE	= 0;
			U1OTGIEbits.ACTVIE = 0;
			IFS5bits.USB1IF	= 0;
		
			//USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
			USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
			//USBSuspendControl	= 0;
		}
	#endif
}
#endif

/******************************************************************************
 * Function:		void USBCBWakeFromSuspend(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		The	host may put USB peripheral	devices	in low power
 *					suspend	mode (by "sending" 3+ms	of idle).  Once	in suspend
 *					mode, the host may wake	the	device back	up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup	from USB suspend 
 *					is detected.
 *
 * Note:			None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock	switching or other power savings measures were taken when
	// executing the USBCBSuspend()	function, now would	be a good time to
	// switch back to normal full power	run	mode conditions.  The host allows
	// a few milliseconds of wakeup	time, after	which the device must be 
	// fully back to normal, and capable of	receiving and processing USB
	// packets.	 In	order to do	this, the USB module must receive proper
	// clocking	(IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:		void USBCB_SOF_Handler(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		The	USB	host sends out a SOF packet	to full-speed
 *					devices	every 1	ms.	This interrupt may be useful
 *					for	isochronous	pipes. End designers should
 *					implement callback routine as necessary.
 *
 * Note:			None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
	// No need to clear	UIRbits.SOFIF to 0 here.
	// Callback	caller is already doing	that.
}

/*******************************************************************
 * Function:		void USBCBErrorHandler(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		The	purpose	of this	callback is	mainly for
 *					debugging during development. Check	UEIR to	see
 *					which error	causes the interrupt.
 *
 * Note:			None
 *******************************************************************/
void USBCBErrorHandler(void)
{
	// No need to clear	UEIR to	0 here.
	// Callback	caller is already doing	that.

	// Typically, user firmware	does not need to do	anything special
	// if a	USB	error occurs.  For example,	if the host	sends an OUT
	// packet to your device, but the packet gets corrupted	(ex:
	// because of a	bad	connection,	or the user	unplugs	the
	// USB cable during	the	transmission) this will	typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to	be done	however, since the SIE will	automatically
	// send	a "NAK"	packet to the host.	 In	response to	this, the
	// host	will normally retry	to send	the	packet again, and no
	// data	loss occurs.  The system will typically	recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function	is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:		void USBCBCheckOtherReq(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		When SETUP packets arrive from the host, some
 *					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some	of
 *					the	SETUP packets will be for standard
 *					USB	"chapter 9"	(as	in,	fulfilling chapter 9 of
 *					the	official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is	being implemented.	For	example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.	This
 *					is not a standard USB chapter 9	request, and 
 *					therefore not handled by usb_device.c.	Instead
 *					this request should	be handled by class	specific 
 *					firmware, such as that contained in	usb_function_hid.c.
 *
 * Note:			None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
	USBCheckCDCRequest();
}//end


/*******************************************************************
 * Function:		void USBCBStdSetDscHandler(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		The	USBCBStdSetDscHandler()	callback function is
 *					called when	a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically	SET_DESCRIPTOR requests	are
 *					not	used in	most applications, and it is
 *					optional to	support	this type of request.
 *
 * Note:			None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
	// Must	claim session ownership	if supporting this request
}//end


/*******************************************************************
 * Function:		void USBCBInitEP(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function is called	when the device	becomes
 *					initialized, which occurs after	the	host sends a
 *					SET_CONFIGURATION (wValue not =	0) request.	 This 
 *					callback function should initialize	the	endpoints 
 *					for	the	device's usage according to	the	current	
 *					configuration.
 *
 * Note:			None
 *******************************************************************/
void USBCBInitEP(void)
{
	CDCInitEP();
}

/********************************************************************
 * Function:		void USBCBSendResume(void)
 *
 * PreCondition:	None
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		The	USB	specifications allow some types	of USB
 *					peripheral devices to wake up a	host PC	(such
 *					as if it is	in a low power suspend to RAM state).
 *					This can be	a very useful feature in some
 *					USB	applications, such as an Infrared remote
 *					control	receiver.  If a	user presses the "power"
 *					button on a	remote control,	it is nice that	the
 *					IR receiver	can	detect this	signalling,	and	then
 *					send a USB "command" to	the	PC to wake up.
 *					
 *					The	USBCBSendResume() "callback" function is used
 *					to send	this special USB signalling	which wakes	
 *					up the PC.	This function may be called	by
 *					application	firmware to	wake up	the	PC.	 This
 *					function should	only be	called when:
 *					
 *					1.	The	USB	driver used	on the host	PC supports
 *						the	remote wakeup capability.
 *					2.	The	USB	configuration descriptor indicates
 *						the	device is remote wakeup	capable	in the
 *						bmAttributes field.
 *					3.	The	USB	host PC	is currently sleeping,
 *						and	has	previously sent	your device	a SET 
 *						FEATURE	setup packet which "armed" the
 *						remote wakeup capability.	
 *
 *					This callback should send a	RESUME signal that
 *					has	the	period of 1-15ms.
 *
 * Note:			Interrupt vs. Polling
 *					-Primary clock
 *					-Secondary clock ***** MAKE	NOTES ABOUT	THIS *******
 *					 > Can switch to primary first by calling USBCBWakeFromSuspend()
 
 *					The	modifiable section in this routine should be changed
 *					to meet	the	application	needs. Current implementation
 *					temporary blocks other functions from executing	for	a
 *					period of 1-13 ms depending	on the core	frequency.
 *
 *					According to USB 2.0 specification section 7.1.7.7,
 *					"The remote wakeup device must hold the resume signaling
 *					for	at lest	1 ms but for no	more than 15 ms."
 *					The	idea here is to	use	a delay	counter	loop, using	a
 *					common value that would	work over a	wide range of core
 *					frequencies.
 *					That value selected	is 1800. See table below:
 *					==========================================================
 *					Core Freq(MHz)		MIP			RESUME Signal Period (ms)
 *					==========================================================
 *						48				12			1.05
 *						 4				1			12.6
 *					==========================================================
 *					* These	timing could be	incorrect when using code
 *					  optimization or extended instruction mode,
 *					  or when having other interrupts enabled.
 *					  Make sure	to verify using	the	MPLAB SIM's	Stopwatch
 *					  and verify the actual	signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
	static WORD	delay_count;
	
	USBResumeControl = 1;				 //	Start RESUME signaling
	
	delay_count	= 1800U;				// Set RESUME line for 1-13	ms
	do
	{
		delay_count--;
	}while(delay_count);
	USBResumeControl = 0;
}


/*******************************************************************
 * Function:		void USBCBEP0DataReceived(void)
 *
 * PreCondition:	ENABLE_EP0_DATA_RECEIVED_CALLBACK must be
 *					defined	already	(in	usb_config.h)
 *
 * Input:			None
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function is called	whenever a EP0 data
 *					packet is received.	 This gives	the	user (and
 *					thus the various class examples	a way to get
 *					data that is received via the control endpoint.
 *					This function needs	to be used in conjunction
 *					with the USBCBCheckOtherReq() function since 
 *					the	USBCBCheckOtherReq() function is the apps
 *					method for getting the initial control transfer
 *					before the data	arrives.
 *
 * Note:			None
 *******************************************************************/
#if	defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived(void)
{
}
#endif

/*******************************************************************
 * Function:		BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *						  USB_EVENT	event, void	*pdata,	WORD size)
 *
 * PreCondition:	None
 *
 * Input:			USB_EVENT event	- the type of event
 *					void *pdata	- pointer to the event data
 *					WORD size -	size of	the	event data
 *
 * Output:			None
 *
 * Side	Effects:	None
 *
 * Overview:		This function is called	from the USB stack to
 *					notify a user application that a USB event
 *					occured.  This callback	is in interrupt	context
 *					when the USB_INTERRUPT option is selected.
 *
 * Note:			None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD	size)
{
	switch(event)
	{
		case EVENT_CONFIGURED: 
			USBCBInitEP();
			break;
		case EVENT_SET_DESCRIPTOR:
			USBCBStdSetDscHandler();
			break;
		case EVENT_EP0_REQUEST:
			USBCBCheckOtherReq();
			break;
		case EVENT_SOF:
			USBCB_SOF_Handler();
			break;
		case EVENT_SUSPEND:
			USBCBSuspend();
			break;
		case EVENT_RESUME:
			USBCBWakeFromSuspend();
			break;
		case EVENT_BUS_ERROR:
			USBCBErrorHandler();
			break;
		case EVENT_TRANSFER:
			Nop();
			break;
		default:
			break;
	}	   
	return TRUE; 
}

/**	EOF	main.c *************************************************/
